@@ -30,7 +30,9 @@
 #include "ide-context-private.h"
 #include "ide-debug.h"
 #include "ide-gsettings-action-group.h"
+#include "ide-log-model-private.h"
 #include "ide-macros.h"
+#include "ide-marshal.h"
 #include "ide-notifications.h"
 #include "ide-settings.h"
 
@@ -50,6 +52,7 @@ struct _IdeContext
   char           *title;
   GFile          *workdir;
   IdeActionMuxer *action_muxer;
+  IdeLogModel    *logs;
   guint           project_loaded : 1;
 };
 
@@ -61,15 +64,9 @@ enum {
   N_PROPS
 };
 
-enum {
-  LOG,
-  N_SIGNALS
-};
-
 G_DEFINE_FINAL_TYPE (IdeContext, ide_context, IDE_TYPE_OBJECT)
 
 static GParamSpec *properties [N_PROPS];
-static guint signals [N_SIGNALS];
 static const char *app_schema_ids[] = {
   "org.gnome.builder",
   "org.gnome.builder.code-insight",
@@ -84,15 +81,6 @@ static const char *project_schema_ids[] = {
   "org.gnome.builder.project",
 };
 
-static void
-ide_context_real_log (IdeContext     *self,
-                      GLogLevelFlags  level,
-                      const gchar    *domain,
-                      const gchar    *message)
-{
-  g_log (domain, level, "%s", message);
-}
-
 static gchar *
 ide_context_repr (IdeObject *object)
 {
@@ -223,6 +211,7 @@ ide_context_finalize (GObject *object)
 
   g_clear_object (&self->action_muxer);
   g_clear_object (&self->workdir);
+  g_clear_object (&self->logs);
   g_clear_pointer (&self->project_id, g_free);
   g_clear_pointer (&self->title, g_free);
 
@@ -347,27 +336,6 @@ ide_context_class_init (IdeContextClass *klass)
                          (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_properties (object_class, N_PROPS, properties);
-
-  /**
-   * IdeContext::log:
-   * @self: an #IdeContext
-   * @severity: the log severity
-   * @domain: the log domain
-   * @message: the log message
-   *
-   * This signal is emitted when a log item has been added for the context.
-   */
-  signals [LOG] =
-    g_signal_new_class_handler ("log",
-                                G_TYPE_FROM_CLASS (klass),
-                                G_SIGNAL_RUN_LAST,
-                                G_CALLBACK (ide_context_real_log),
-                                NULL, NULL, NULL,
-                                G_TYPE_NONE,
-                                3,
-                                G_TYPE_UINT,
-                                G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
-                                G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 }
 
 static void
@@ -382,6 +350,8 @@ ide_context_init (IdeContext *self)
 
   notifs = ide_notifications_new ();
   ide_object_append (IDE_OBJECT (self), IDE_OBJECT (notifs));
+
+  self->logs = _ide_log_model_new ();
 }
 
 /**
@@ -501,10 +471,8 @@ ide_context_set_project_id (IdeContext *self,
     project_id = "empty";
 
   ide_object_lock (IDE_OBJECT (self));
-  if (!ide_str_equal0 (self->project_id, project_id))
+  if (g_set_str (&self->project_id, project_id))
     {
-      g_free (self->project_id);
-      self->project_id = g_strdup (project_id);
       ide_object_notify_by_pspec (IDE_OBJECT (self), properties [PROP_PROJECT_ID]);
       register_settings = TRUE;
     }
@@ -843,10 +811,8 @@ ide_context_set_title (IdeContext  *self,
     title = _("Untitled");
 
   ide_object_lock (IDE_OBJECT (self));
-  if (!ide_str_equal0 (self->title, title))
+  if (g_set_str (&self->title, title))
     {
-      g_free (self->title);
-      self->title = g_strdup (title);
       ide_object_notify_by_pspec (IDE_OBJECT (self), properties [PROP_TITLE]);
     }
   ide_object_unlock (IDE_OBJECT (self));
@@ -860,7 +826,10 @@ ide_context_log (IdeContext     *self,
 {
   g_assert (IDE_IS_CONTEXT (self));
 
-  g_signal_emit (self, signals [LOG], 0, level, domain, message);
+  if (self->logs != NULL)
+    _ide_log_model_append (self->logs, level, domain, message);
+
+  g_log (domain, level, "%s", message);
 }
 
 /**
@@ -967,3 +936,21 @@ ide_context_unregister_settings (IdeContext *self,
       ide_action_muxer_insert_action_group (muxer, app_group, NULL);
     }
 }
+
+/**
+ * ide_context_ref_logs:
+ * @self: a #IdeContext
+ *
+ * Gets the logs for the context.
+ *
+ * Returns: (transfer full): a #GListModel of #IdeLogItem
+ *
+ * Since: 44
+ */
+GListModel *
+ide_context_ref_logs (IdeContext *self)
+{
+  g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
+
+  return g_object_ref (G_LIST_MODEL (self->logs));
+}
