@@ -98,21 +98,41 @@ ide_editor_page_notify_file_cb (IdeEditorPage *self,
                            g_object_ref (self));
 }
 
+static void
+ide_editor_page_update_actions (IdeEditorPage *self)
+{
+  IdeFormatter *formatter;
+  gboolean has_selection;
+
+  g_assert (IDE_IS_EDITOR_PAGE (self));
+
+  formatter = ide_buffer_get_formatter (self->buffer);
+  has_selection = gtk_text_buffer_get_has_selection (GTK_TEXT_BUFFER (self->buffer));
+
+  panel_widget_action_set_enabled (PANEL_WIDGET (self), "editor.format-document", formatter && !has_selection);
+  panel_widget_action_set_enabled (PANEL_WIDGET (self), "editor.format-selection", formatter && has_selection);
+}
+
 static void
 ide_editor_page_notify_formatter_cb (IdeEditorPage *self,
                                      GParamSpec    *pspec,
                                      IdeBuffer     *buffer)
 {
-  IdeFormatter *formatter;
-
   g_assert (IDE_IS_EDITOR_PAGE (self));
   g_assert (IDE_IS_BUFFER (buffer));
 
-  formatter = ide_buffer_get_formatter (buffer);
+  ide_editor_page_update_actions (self);
+}
 
-  panel_widget_action_set_enabled (PANEL_WIDGET (self),
-                                   "editor.format",
-                                   formatter != NULL);
+static void
+ide_editor_page_notify_has_selection_cb (IdeEditorPage *self,
+                                         GParamSpec    *pspec,
+                                         IdeBuffer     *buffer)
+{
+  g_assert (IDE_IS_EDITOR_PAGE (self));
+  g_assert (IDE_IS_BUFFER (buffer));
+
+  ide_editor_page_update_actions (self);
 }
 
 static void
@@ -142,6 +162,17 @@ ide_editor_page_style_scheme_changed_cb (IdeEditorPage *self,
     ide_gutter_style_changed (self->gutter);
 }
 
+static gboolean
+file_to_basename (GBinding     *binding,
+                  const GValue *from_value,
+                  GValue       *to_value,
+                  gpointer      user_data)
+{
+  GFile *file = g_value_get_object (from_value);
+  g_value_take_string (to_value, g_file_get_basename (file));
+  return TRUE;
+}
+
 static void
 ide_editor_page_set_buffer (IdeEditorPage *self,
                             IdeBuffer     *buffer)
@@ -175,6 +206,12 @@ ide_editor_page_set_buffer (IdeEditorPage *self,
                                self,
                                G_CONNECT_SWAPPED);
 
+      g_signal_connect_object (buffer,
+                               "notify::has-selection",
+                               G_CALLBACK (ide_editor_page_notify_has_selection_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
+
       g_signal_connect_object (buffer,
                                "notify::file-settings",
                                G_CALLBACK (_ide_editor_page_settings_reload),
@@ -187,8 +224,13 @@ ide_editor_page_set_buffer (IdeEditorPage *self,
                                self,
                                G_CONNECT_SWAPPED);
 
+      g_object_bind_property_full (buffer, "file",
+                                   self, "title",
+                                   G_BINDING_SYNC_CREATE,
+                                   file_to_basename, NULL,
+                                   NULL, NULL);
       g_object_bind_property (buffer, "title",
-                              self, "title",
+                              self, "tooltip",
                               G_BINDING_SYNC_CREATE);
 
       ide_editor_page_notify_file_cb (self, NULL, buffer);
@@ -430,6 +472,26 @@ search_begin_replace_action (GtkWidget  *widget,
   set_search_visible (IDE_EDITOR_PAGE (widget), TRUE, IDE_EDITOR_SEARCH_BAR_MODE_REPLACE);
 }
 
+static void
+search_move_next_action (GtkWidget  *widget,
+                         const char *action_name,
+                         GVariant   *param)
+{
+  IdeEditorPage *self = IDE_EDITOR_PAGE (widget);
+
+  _ide_editor_search_bar_move_next (self->search_bar, FALSE);
+}
+
+static void
+search_move_previous_action (GtkWidget  *widget,
+                             const char *action_name,
+                             GVariant   *param)
+{
+  IdeEditorPage *self = IDE_EDITOR_PAGE (widget);
+
+  _ide_editor_search_bar_move_previous (self->search_bar, FALSE);
+}
+
 static void
 handle_print_result (IdeEditorPage           *self,
                      GtkPrintOperation       *operation,
@@ -599,6 +661,122 @@ reload_action (GtkWidget  *widget,
   IDE_EXIT;
 }
 
+static void
+next_diagnostic_cb (guint                 line,
+                    IdeDiagnosticSeverity severity,
+                    gpointer              user_data)
+{
+  int *out_line = user_data;
+
+  if (*out_line == -1)
+    *out_line = line;
+}
+
+static void
+diagnostics_next_action (GtkWidget  *widget,
+                         const char *action_name,
+                         GVariant   *param)
+{
+  IdeEditorPage *self = (IdeEditorPage *)widget;
+  IdeDiagnostics *diagnostics;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_EDITOR_PAGE (self));
+
+  if ((diagnostics = ide_buffer_get_diagnostics (self->buffer)))
+    {
+      GtkTextIter insert;
+      GtkTextIter begin, end;
+      int line = -1;
+
+      ide_buffer_get_selection_bounds (self->buffer, &insert, NULL);
+      gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (self->buffer), &begin, &end);
+
+      ide_diagnostics_foreach_line_in_range (diagnostics,
+                                             ide_buffer_get_file (self->buffer),
+                                             gtk_text_iter_get_line (&insert) + 1,
+                                             gtk_text_iter_get_line (&end),
+                                             next_diagnostic_cb,
+                                             &line);
+
+      if (line == -1)
+        ide_diagnostics_foreach_line_in_range (diagnostics,
+                                               ide_buffer_get_file (self->buffer),
+                                               0,
+                                               gtk_text_iter_get_line (&insert),
+                                               next_diagnostic_cb,
+                                               &line);
+
+      if (line != -1)
+        {
+          gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (self->buffer), &insert, line);
+          gtk_text_buffer_select_range (GTK_TEXT_BUFFER (self->buffer), &insert, &insert);
+          ide_editor_page_scroll_to_insert (self, GTK_DIR_DOWN);
+        }
+    }
+
+  IDE_EXIT;
+}
+
+static void
+previous_diagnostic_cb (guint                 line,
+                        IdeDiagnosticSeverity severity,
+                        gpointer              user_data)
+{
+  int *out_line = user_data;
+  *out_line = line;
+}
+
+static void
+diagnostics_previous_action (GtkWidget  *widget,
+                             const char *action_name,
+                             GVariant   *param)
+{
+  IdeEditorPage *self = (IdeEditorPage *)widget;
+  IdeDiagnostics *diagnostics;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_EDITOR_PAGE (self));
+
+  if ((diagnostics = ide_buffer_get_diagnostics (self->buffer)))
+    {
+      GtkTextIter insert;
+      GtkTextIter begin, end;
+      int line = -1;
+
+      ide_buffer_get_selection_bounds (self->buffer, &insert, NULL);
+      gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (self->buffer), &begin, &end);
+
+      ide_diagnostics_foreach_line_in_range (diagnostics,
+                                             ide_buffer_get_file (self->buffer),
+                                             0,
+                                             MAX (1, gtk_text_iter_get_line (&insert)) - 1,
+                                             previous_diagnostic_cb,
+                                             &line);
+
+      if (line == -1)
+        ide_diagnostics_foreach_line_in_range (diagnostics,
+                                               ide_buffer_get_file (self->buffer),
+                                               gtk_text_iter_get_line (&insert),
+                                               gtk_text_iter_get_line (&end),
+                                               previous_diagnostic_cb,
+                                               &line);
+
+      if (line != -1)
+        {
+          gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (self->buffer), &insert, line);
+          gtk_text_buffer_select_range (GTK_TEXT_BUFFER (self->buffer), &insert, &insert);
+          ide_editor_page_scroll_to_insert (self, GTK_DIR_UP);
+        }
+    }
+
+  IDE_EXIT;
+}
+
 static void
 ide_editor_page_constructed (GObject *object)
 {
@@ -752,9 +930,14 @@ ide_editor_page_class_init (IdeEditorPageClass *klass)
   panel_widget_class_install_action (panel_widget_class, "search.hide", NULL, search_hide_action);
   panel_widget_class_install_action (panel_widget_class, "search.begin-find", NULL, search_begin_find_action);
   panel_widget_class_install_action (panel_widget_class, "search.begin-replace", NULL, search_begin_replace_action);
+  panel_widget_class_install_action (panel_widget_class, "search.move-next", NULL, search_move_next_action);
+  panel_widget_class_install_action (panel_widget_class, "search.move-previous", NULL, search_move_previous_action);
   panel_widget_class_install_action (panel_widget_class, "editor.print", NULL, print_action);
-  panel_widget_class_install_action (panel_widget_class, "editor.format", NULL, format_action);
+  panel_widget_class_install_action (panel_widget_class, "editor.format-document", NULL, format_action);
+  panel_widget_class_install_action (panel_widget_class, "editor.format-selection", NULL, format_action);
   panel_widget_class_install_action (panel_widget_class, "editor.reload", NULL, reload_action);
+  panel_widget_class_install_action (panel_widget_class, "editor.diagnostics.next", NULL, diagnostics_next_action);
+  panel_widget_class_install_action (panel_widget_class, "editor.diagnostics.previous", NULL, diagnostics_previous_action);
 
   g_type_ensure (IDE_TYPE_EDITOR_INFO_BAR);
   g_type_ensure (IDE_TYPE_EDITOR_SEARCH_BAR);
@@ -1160,9 +1343,10 @@ ide_editor_page_scroll_to_visual_position (IdeEditorPage *self,
 }
 
 void
-ide_editor_page_scroll_to_insert (IdeEditorPage *self)
+ide_editor_page_scroll_to_insert (IdeEditorPage    *self,
+                                  GtkDirectionType  dir)
 {
   g_return_if_fail (IDE_IS_EDITOR_PAGE (self));
 
-  ide_source_view_scroll_to_insert (self->view);
+  ide_source_view_scroll_to_insert (self->view, dir);
 }
