@@ -23,6 +23,7 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
+
 #include <libpeas/peas.h>
 
 #include <libide-core.h>
@@ -31,6 +32,8 @@
 #include <libide-threading.h>
 #include <libide-vcs.h>
 
+#include "ide-marshal.h"
+
 #include "ide-build-manager.h"
 #include "ide-build-private.h"
 #include "ide-build-target.h"
@@ -129,6 +132,8 @@ static void ide_build_manager_action_export               (IdeBuildManager *self
                                                            GVariant        *param);
 static void ide_build_manager_action_install              (IdeBuildManager *self,
                                                            GVariant        *param);
+static void ide_build_manager_action_invalidate           (IdeBuildManager *self,
+                                                           GVariant        *param);
 static void ide_build_manager_action_default_build_target (IdeBuildManager *self,
                                                            GVariant        *param);
 
@@ -140,6 +145,7 @@ IDE_DEFINE_ACTION_GROUP (IdeBuildManager, ide_build_manager, {
   { "install", ide_build_manager_action_install },
   { "rebuild", ide_build_manager_action_rebuild },
   { "default-build-target", ide_build_manager_action_default_build_target, "s", "''" },
+  { "invalidate", ide_build_manager_action_invalidate },
 })
 
 G_DEFINE_TYPE_EXTENDED (IdeBuildManager, ide_build_manager, IDE_TYPE_OBJECT, G_TYPE_FLAG_FINAL,
@@ -180,6 +186,16 @@ build_state_free (BuildState *state)
   g_slice_free (BuildState, state);
 }
 
+static void
+ide_build_manager_action_invalidate (IdeBuildManager *self,
+                                     GVariant        *param)
+{
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_BUILD_MANAGER (self));
+
+  ide_build_manager_invalidate (self);
+}
+
 static void
 ide_build_manager_action_default_build_target (IdeBuildManager *self,
                                                GVariant        *param)
@@ -194,14 +210,10 @@ ide_build_manager_action_default_build_target (IdeBuildManager *self,
   if (ide_str_empty0 (str))
     str = NULL;
 
-  if (g_strcmp0 (str, self->default_build_target) != 0)
-    {
-      g_free (self->default_build_target);
-      self->default_build_target = g_strdup (str);
-      ide_build_manager_set_action_state (self,
-                                          "default-build-target",
-                                          g_variant_new_string (str ? str : ""));
-    }
+  if (g_set_str (&self->default_build_target, str))
+    ide_build_manager_set_action_state (self,
+                                        "default-build-target",
+                                        g_variant_new_string (str ? str : ""));
 }
 
 static void
@@ -643,6 +655,12 @@ ide_build_manager_invalidate_pipeline (IdeBuildManager *self)
 
   g_assert (IDE_IS_BUILD_MANAGER (self));
 
+  if (!self->started)
+    {
+      IDE_TRACE_MSG ("Ignoring invalidation, project not yet loaded");
+      IDE_EXIT;
+    }
+
   context = ide_object_get_context (IDE_OBJECT (self));
 
   IDE_TRACE_MSG ("Reloading pipeline due to configuration change");
@@ -746,12 +764,8 @@ ide_build_manager_vcs_changed (IdeBuildManager *self,
 
   branch_name = ide_vcs_get_branch_name (vcs);
 
-  if (!ide_str_equal0 (branch_name, self->branch_name))
-    {
-      g_free (self->branch_name);
-      self->branch_name = g_strdup (branch_name);
-      ide_build_manager_invalidate_pipeline (self);
-    }
+  if (g_set_str (&self->branch_name, branch_name))
+    ide_build_manager_invalidate_pipeline (self);
 }
 
 static gboolean
@@ -1083,8 +1097,11 @@ ide_build_manager_class_init (IdeBuildManagerClass *klass)
                                 G_SIGNAL_RUN_LAST,
                                 G_CALLBACK (ide_build_manager_real_build_started),
                                 NULL, NULL,
-                                NULL,
+                                ide_marshal_VOID__OBJECT,
                                 G_TYPE_NONE, 1, IDE_TYPE_PIPELINE);
+  g_signal_set_va_marshaller (signals [BUILD_STARTED],
+                              G_TYPE_FROM_CLASS (klass),
+                              ide_marshal_VOID__OBJECTv);
 
   /**
    * IdeBuildManager::build-failed:
@@ -1104,8 +1121,11 @@ ide_build_manager_class_init (IdeBuildManagerClass *klass)
                                 G_SIGNAL_RUN_LAST,
                                 G_CALLBACK (ide_build_manager_real_build_failed),
                                 NULL, NULL,
-                                NULL,
+                                ide_marshal_VOID__OBJECT,
                                 G_TYPE_NONE, 1, IDE_TYPE_PIPELINE);
+  g_signal_set_va_marshaller (signals [BUILD_FAILED],
+                              G_TYPE_FROM_CLASS (klass),
+                              ide_marshal_VOID__OBJECTv);
 
   /**
    * IdeBuildManager::build-finished:
@@ -1121,8 +1141,11 @@ ide_build_manager_class_init (IdeBuildManagerClass *klass)
                                 G_SIGNAL_RUN_LAST,
                                 G_CALLBACK (ide_build_manager_real_build_finished),
                                 NULL, NULL,
-                                NULL,
+                                ide_marshal_VOID__OBJECT,
                                 G_TYPE_NONE, 1, IDE_TYPE_PIPELINE);
+  g_signal_set_va_marshaller (signals [BUILD_FINISHED],
+                              G_TYPE_FROM_CLASS (klass),
+                              ide_marshal_VOID__OBJECTv);
 }
 
 static void
@@ -1416,7 +1439,8 @@ ide_build_manager_build_targets_cb (GObject      *object,
 
   if (!ide_pipeline_build_targets_finish (pipeline, result, &error))
     {
-      ide_object_warning (pipeline, "%s", error->message);
+      if (!ide_error_ignore (error))
+        ide_object_warning (pipeline, "%s", error->message);
       ide_task_return_error (task, g_steal_pointer (&error));
       IDE_GOTO (failure);
     }
@@ -1940,10 +1964,14 @@ ide_build_manager_set_can_build (IdeBuildManager *self,
 void
 ide_build_manager_invalidate (IdeBuildManager *self)
 {
+  IDE_ENTRY;
+
   g_return_if_fail (IDE_IS_MAIN_THREAD ());
   g_return_if_fail (IDE_IS_BUILD_MANAGER (self));
 
   ide_build_manager_invalidate_pipeline (self);
+
+  IDE_EXIT;
 }
 
 guint
