@@ -23,10 +23,13 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
+
 #include <libpeas/peas.h>
 
 #include <libide-threading.h>
 
+#include "ide-marshal.h"
+
 #include "ide-config.h"
 #include "ide-config-manager.h"
 #include "ide-config-private.h"
@@ -36,7 +39,7 @@
 
 struct _IdeConfigManager
 {
-  GObject           parent_instance;
+  IdeObject         parent_instance;
 
   GCancellable     *cancellable;
   GArray           *configs;
@@ -50,6 +53,7 @@ struct _IdeConfigManager
   guint             queued_save_source;
 
   guint             propagate_to_settings : 1;
+  guint             save_needs_invalidate : 1;
 };
 
 typedef struct
@@ -336,7 +340,7 @@ ide_config_manager_notify_ready (IdeConfigManager *self,
 }
 
 static void
-ide_config_manager_dispose (GObject *object)
+ide_config_manager_destroy (IdeObject *object)
 {
   IdeConfigManager *self = (IdeConfigManager *)object;
 
@@ -353,7 +357,7 @@ ide_config_manager_dispose (GObject *object)
   g_cancellable_cancel (self->cancellable);
   g_clear_object (&self->project_settings);
 
-  G_OBJECT_CLASS (ide_config_manager_parent_class)->dispose (object);
+  IDE_OBJECT_CLASS (ide_config_manager_parent_class)->destroy (object);
 }
 
 static void
@@ -471,12 +475,14 @@ static void
 ide_config_manager_class_init (IdeConfigManagerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  IdeObjectClass *i_object_class = IDE_OBJECT_CLASS (klass);
 
-  object_class->dispose = ide_config_manager_dispose;
   object_class->finalize = ide_config_manager_finalize;
   object_class->get_property = ide_config_manager_get_property;
   object_class->set_property = ide_config_manager_set_property;
 
+  i_object_class->destroy = ide_config_manager_destroy;
+
   properties [PROP_CURRENT] =
     g_param_spec_object ("current",
                          "Current",
@@ -512,8 +518,11 @@ ide_config_manager_class_init (IdeConfigManagerClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   0, NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  ide_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+  g_signal_set_va_marshaller (signals [INVALIDATE],
+                              G_TYPE_FROM_CLASS (klass),
+                              ide_marshal_VOID__VOIDv);
 }
 
 static void
@@ -586,7 +595,11 @@ ide_config_manager_do_save (gpointer data)
 
   self->queued_save_source = 0;
 
-  g_signal_emit (self, signals [INVALIDATE], 0);
+  if (self->save_needs_invalidate)
+    {
+      self->save_needs_invalidate = FALSE;
+      g_signal_emit (self, signals [INVALIDATE], 0);
+    }
 
   ide_config_manager_save_async (self, NULL, NULL, NULL);
 
@@ -600,7 +613,21 @@ ide_config_manager_changed (IdeConfigManager *self,
   g_assert (IDE_IS_CONFIG_MANAGER (self));
   g_assert (IDE_IS_CONFIG (config));
 
-  g_clear_handle_id (&self->queued_save_source, g_source_remove);
+  if (self->queued_save_source != 0)
+    return;
+
+  /* We only care if the changed causes the config to become dirty
+   * and therefore needs a writeback.
+   */
+  if (!ide_config_get_dirty (config))
+    return;
+
+  ide_object_message (self,
+                      _("Configuration %s changed"),
+                      ide_config_get_display_name (config));
+
+  self->save_needs_invalidate |= config == self->current;
+
   self->queued_save_source =
     g_timeout_add_seconds_full (G_PRIORITY_LOW,
                                 WRITEBACK_DELAY_SEC,
@@ -826,14 +853,25 @@ notify_providers_loaded (IdeConfigManager *self,
 
   if (user_value != NULL)
     {
-      const gchar *str = g_variant_get_string (user_value, NULL);
+      const char *config_id = g_variant_get_string (user_value, NULL);
       IdeConfig *config;
 
-      if ((config = ide_config_manager_get_config (self, str)))
+      if ((config = ide_config_manager_get_config (self, config_id)))
         {
           if (config != self->current)
             ide_config_manager_set_current (self, config);
         }
+      else
+        {
+          /* We failed to locate the user's config-id that we last used.
+           * Notify the user so they have some sort of indication of
+           * build pipeline failure.
+           */
+          ide_object_message (IDE_OBJECT (self),
+                              /* translators: %s will be replaced with the build configuration identifier */
+                              _("Failed to locate build configuration “%s”. It may be invalid or incorrectly formatted."),
+                              config_id);
+        }
     }
 
   self->propagate_to_settings = TRUE;
@@ -1025,6 +1063,11 @@ ide_config_manager_set_current (IdeConfigManager *self,
 
       ide_config_manager_set_action_state (self, "current", g_variant_new_string (id));
 
+      ide_object_message (IDE_OBJECT (self),
+                          /* translators: %s is set to the identifier of the build configuration */
+                          _("Active configuration set to “%s”"),
+                          id);
+
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_CURRENT]);
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_CURRENT_DISPLAY_NAME]);
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_READY]);
