@@ -57,7 +57,13 @@ enum {
   N_PROPS
 };
 
+enum {
+  SHOW_HELP,
+  N_SIGNALS
+};
+
 static GParamSpec *properties[N_PROPS];
+static guint signals[N_SIGNALS];
 
 static void
 ide_application_add_platform_data (GApplication    *app,
@@ -427,6 +433,21 @@ _ide_application_remove_resources (IdeApplication *self,
   g_warning ("TODO: implement resource unloading for plugins: %s", resource_path);
 }
 
+static gboolean
+ide_application_show_help_external (IdeApplication *self)
+{
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_APPLICATION (self));
+
+  gtk_show_uri (NULL,
+                "https://builder.readthedocs.io",
+                GDK_CURRENT_TIME);
+
+  IDE_RETURN (TRUE);
+}
+
 static void
 ide_application_get_property (GObject    *object,
                               guint       prop_id,
@@ -542,6 +563,15 @@ ide_application_class_init (IdeApplicationClass *klass)
                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_properties (object_class, N_PROPS, properties);
+
+  signals [SHOW_HELP] =
+    g_signal_new_class_handler ("show-help",
+                                G_TYPE_FROM_CLASS (klass),
+                                G_SIGNAL_RUN_LAST,
+                                G_CALLBACK (ide_application_show_help_external),
+                                g_signal_accumulator_true_handled, NULL,
+                                NULL,
+                                G_TYPE_BOOLEAN, 0);
 }
 
 static void
@@ -747,17 +777,6 @@ ide_application_has_network (IdeApplication *self)
                                G_CONNECT_SWAPPED);
 
       self->has_network = g_network_monitor_get_network_available (self->network_monitor);
-
-      /*
-       * FIXME: Ignore the network portal initially for now.
-       *
-       * See https://gitlab.gnome.org/GNOME/glib/merge_requests/227 for more
-       * information about when this is fixed.
-       *
-       * See Also: https://gitlab.gnome.org/GNOME/glib/-/issues/1718
-       */
-      if (!self->has_network && ide_is_flatpak ())
-        self->has_network = TRUE;
     }
 
   return self->has_network;
@@ -946,6 +965,15 @@ get_user_style_file (GFile *file)
 
   basename = g_file_get_basename (file);
 
+  /* Style schemes must have .xml suffix to be picked up
+   * by GtkSourceView. See GNOME/gnome-builder#1999.
+   */
+  if (!g_str_has_suffix (basename, ".xml"))
+    {
+      g_autofree char *tmp = g_steal_pointer (&basename);
+      basename = g_strdup_printf ("%s.xml", tmp);
+    }
+
   return g_file_get_child (style_dir, basename);
 }
 
@@ -961,6 +989,8 @@ ide_application_install_schemes_cb (GObject      *object,
   GPtrArray *ar;
   GFile *src;
 
+  IDE_ENTRY;
+
   g_assert (G_IS_FILE (file));
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (G_IS_TASK (task));
@@ -979,7 +1009,7 @@ ide_application_install_schemes_cb (GObject      *object,
   if (ar->len == 0)
     {
       g_task_return_boolean (task, TRUE);
-      return;
+      IDE_EXIT;
     }
 
   src = g_ptr_array_index (ar, ar->len-1);
@@ -992,6 +1022,8 @@ ide_application_install_schemes_cb (GObject      *object,
                      NULL, NULL,
                      ide_application_install_schemes_cb,
                      g_object_ref (task));
+
+  IDE_EXIT;
 }
 
 void
@@ -1009,6 +1041,8 @@ ide_application_install_schemes_async (IdeApplication       *self,
   g_autoptr(GFile) dir = NULL;
   GFile *src;
 
+  IDE_ENTRY;
+
   g_return_if_fail (IDE_IS_MAIN_THREAD ());
   g_return_if_fail (IDE_IS_APPLICATION (self));
   g_return_if_fail (files != NULL);
@@ -1030,8 +1064,9 @@ ide_application_install_schemes_async (IdeApplication       *self,
   if (!g_file_query_exists (dir, NULL) &&
       !g_file_make_directory_with_parents (dir, cancellable, &error))
     {
+      g_warning ("Failed to create directory for style scheme: %s", error->message);
       g_task_return_error (task, g_steal_pointer (&error));
-      return;
+      IDE_EXIT;
     }
 
   g_file_copy_async (src, dst,
@@ -1041,6 +1076,8 @@ ide_application_install_schemes_async (IdeApplication       *self,
                      NULL, NULL,
                      ide_application_install_schemes_cb,
                      g_steal_pointer (&task));
+
+  IDE_EXIT;
 }
 
 gboolean
@@ -1053,3 +1090,36 @@ ide_application_install_schemes_finish (IdeApplication  *self,
 
   return g_task_propagate_boolean (G_TASK (result), error);
 }
+
+/**
+ * ide_application_find_project_workbench:
+ * @self: a #IdeApplication
+ * @project_info: an #IdeProjectInfo
+ *
+ * Finds a workbench that has @project_info loaded.
+ *
+ * If no workbench could be found, %NULL is returned.
+ *
+ * Returns: (transfer none) (nullable): an #IdeWorkbench or %NULL
+ *
+ * Since: 44
+ */
+IdeWorkbench *
+ide_application_find_project_workbench (IdeApplication *self,
+                                        IdeProjectInfo *project_info)
+{
+  g_return_val_if_fail (IDE_IS_APPLICATION (self), NULL);
+  g_return_val_if_fail (IDE_IS_PROJECT_INFO (project_info), NULL);
+
+  for (guint i = 0; i < self->workbenches->len; i++)
+    {
+      IdeWorkbench *workbench = g_ptr_array_index (self->workbenches, i);
+      IdeProjectInfo *workbench_project_info = ide_workbench_get_project_info (workbench);
+
+      if (workbench_project_info != NULL &&
+          ide_project_info_equal (workbench_project_info, project_info))
+        return workbench;
+    }
+
+  return NULL;
+}
