@@ -115,6 +115,7 @@ enum {
   PROP_0,
   PROP_CONTEXT,
   PROP_ID,
+  PROP_SEARCH_POPOVER,
   N_PROPS
 };
 
@@ -626,6 +627,7 @@ ide_workspace_finalize (GObject *object)
   IdeWorkspace *self = (IdeWorkspace *)object;
   IdeWorkspacePrivate *priv = ide_workspace_get_instance_private (self);
 
+  g_clear_pointer (&priv->id, g_free);
   g_clear_object (&priv->context);
   g_clear_object (&priv->cancellable);
   g_clear_handle_id (&priv->queued_window_save, g_source_remove);
@@ -640,6 +642,7 @@ ide_workspace_get_property (GObject    *object,
                             GParamSpec *pspec)
 {
   IdeWorkspace *self = IDE_WORKSPACE (object);
+  IdeWorkspacePrivate *priv = ide_workspace_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -651,6 +654,10 @@ ide_workspace_get_property (GObject    *object,
       g_value_set_string (value, ide_workspace_get_id (self));
       break;
 
+    case PROP_SEARCH_POPOVER:
+      g_value_set_object (value, priv->search_popover);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -729,6 +736,11 @@ ide_workspace_class_init (IdeWorkspaceClass *klass)
                          NULL,
                          (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
+  properties [PROP_SEARCH_POPOVER] =
+    g_param_spec_object ("search-popover", NULL, NULL,
+                         IDE_TYPE_SEARCH_POPOVER,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_properties (object_class, N_PROPS, properties);
 
   ide_action_mixin_init (&klass->action_mixin, object_class);
@@ -1488,7 +1500,7 @@ static gboolean
 shortcut_phase_filter (gpointer item,
                        gpointer user_data)
 {
-  return ide_shortcut_is_phase (item, user_data);
+  return ide_shortcut_is_phase (item, GPOINTER_TO_UINT (user_data));
 }
 
 void
@@ -1727,7 +1739,7 @@ ide_workspace_set_id (IdeWorkspace *self,
 
   g_return_if_fail (IDE_IS_WORKSPACE (self));
 
-  if (ide_set_string (&priv->id, id))
+  if (g_set_str (&priv->id, id))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_ID]);
 }
 
@@ -1770,3 +1782,21 @@ _ide_workspace_get_addins (IdeWorkspace *self)
 
   return priv->addins;
 }
+
+gboolean
+_ide_workspace_adopt_widget (IdeWorkspace *workspace,
+                             PanelWidget  *widget,
+                             PanelDock    *dock)
+{
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_WORKSPACE (workspace));
+  g_assert (PANEL_IS_WIDGET (widget));
+  g_assert (PANEL_IS_DOCK (dock));
+
+  if (ide_widget_get_context (GTK_WIDGET (workspace)) == ide_widget_get_context (GTK_WIDGET (widget)))
+    IDE_RETURN (GDK_EVENT_PROPAGATE);
+
+  IDE_RETURN (GDK_EVENT_STOP);
+}
