@@ -28,7 +28,7 @@ G_BEGIN_DECLS
 
 struct _IdeTerminalSearch
 {
-  AdwBin               parent_instance;
+  GtkWidget            parent_instance;
 
   VteTerminal         *terminal;
 
@@ -36,23 +36,22 @@ struct _IdeTerminalSearch
 
   IdeSearchEntry      *search_entry;
 
+  GtkGrid             *grid;
   GtkButton           *search_prev_button;
   GtkButton           *search_next_button;
   GtkButton           *close_button;
 
-  GtkGrid             *search_options;
-
-  GtkToggleButton     *reveal_button;
-  GtkToggleButton     *match_case_checkbutton;
-  GtkToggleButton     *entire_word_checkbutton;
-  GtkToggleButton     *regex_checkbutton;
-  GtkToggleButton     *wrap_around_checkbutton;
-
   /* Cached regex */
   gboolean             regex_caseless;
   gchar               *regex_pattern;
   VteRegex            *regex;
 
+  /* Search options */
+  guint                use_regex : 1;
+  guint                wrap_word : 1;
+  guint                match_case : 1;
+  guint                entire_word : 1;
+
   gchar               *selected_text;
   gchar               *selection_buffer;
 };
