@@ -99,7 +99,7 @@ ide_vcs_clone_request_set_cloner (IdeVcsCloneRequest *self,
 }
 
 static void
-ide_vcs_clone_request_dispose (GObject *object)
+ide_vcs_clone_request_destroy (IdeObject *object)
 {
   IdeVcsCloneRequest *self = (IdeVcsCloneRequest *)object;
 
@@ -115,7 +115,7 @@ ide_vcs_clone_request_dispose (GObject *object)
   ide_clear_string (&self->module_name);
   ide_clear_string (&self->uri);
 
-  G_OBJECT_CLASS (ide_vcs_clone_request_parent_class)->dispose (object);
+  IDE_OBJECT_CLASS (ide_vcs_clone_request_parent_class)->destroy (object);
 }
 
 static void
@@ -212,11 +212,13 @@ static void
 ide_vcs_clone_request_class_init (IdeVcsCloneRequestClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  IdeObjectClass *i_object_class = IDE_OBJECT_CLASS (klass);
 
-  object_class->dispose = ide_vcs_clone_request_dispose;
   object_class->get_property = ide_vcs_clone_request_get_property;
   object_class->set_property = ide_vcs_clone_request_set_property;
 
+  i_object_class->destroy = ide_vcs_clone_request_destroy;
+
   properties [PROP_AUTHOR_EMAIL] =
     g_param_spec_string ("author-email", NULL, NULL, NULL,
                          (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
@@ -352,7 +354,7 @@ ide_vcs_clone_request_set_author_email (IdeVcsCloneRequest *self,
 {
   g_return_if_fail (IDE_IS_VCS_CLONE_REQUEST (self));
 
-  if (ide_set_string (&self->author_email, author_email))
+  if (g_set_str (&self->author_email, author_email))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_AUTHOR_EMAIL]);
 }
 
@@ -362,7 +364,7 @@ ide_vcs_clone_request_set_author_name (IdeVcsCloneRequest *self,
 {
   g_return_if_fail (IDE_IS_VCS_CLONE_REQUEST (self));
 
-  if (ide_set_string (&self->author_name, author_name))
+  if (g_set_str (&self->author_name, author_name))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_AUTHOR_NAME]);
 }
 
@@ -372,7 +374,7 @@ ide_vcs_clone_request_set_branch_name (IdeVcsCloneRequest *self,
 {
   g_return_if_fail (IDE_IS_VCS_CLONE_REQUEST (self));
 
-  if (ide_set_string (&self->branch_name, branch_name))
+  if (g_set_str (&self->branch_name, branch_name))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_BRANCH_NAME]);
 }
 
@@ -382,7 +384,7 @@ ide_vcs_clone_request_set_module_name (IdeVcsCloneRequest *self,
 {
   g_return_if_fail (IDE_IS_VCS_CLONE_REQUEST (self));
 
-  if (ide_set_string (&self->module_name, module_name))
+  if (g_set_str (&self->module_name, module_name))
     {
       g_autoptr(PeasExtension) exten = NULL;
 
@@ -432,7 +434,7 @@ ide_vcs_clone_request_set_uri (IdeVcsCloneRequest *self,
 {
   g_return_if_fail (IDE_IS_VCS_CLONE_REQUEST (self));
 
-  if (ide_set_string (&self->uri, uri))
+  if (g_set_str (&self->uri, uri))
     {
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_CAN_SELECT_BRANCH]);
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_URI]);
@@ -599,6 +601,7 @@ ide_vcs_clone_request_clone_cb (GObject      *object,
 void
 ide_vcs_clone_request_clone_async (IdeVcsCloneRequest  *self,
                                    IdeNotification     *notif,
+                                   int                  pty_fd,
                                    GCancellable        *cancellable,
                                    GAsyncReadyCallback  callback,
                                    gpointer             user_data)
@@ -646,6 +649,8 @@ ide_vcs_clone_request_clone_async (IdeVcsCloneRequest  *self,
   if (!ide_str_empty0 (self->branch_name))
     g_variant_dict_insert (&params, "branch", "s", self->branch_name);
 
+  ide_vcs_cloner_set_pty_fd (self->cloner, pty_fd);
+
   ide_vcs_cloner_clone_async (self->cloner,
                               uri_str,
                               g_file_peek_path (clone_dir),
