@@ -22,6 +22,8 @@
 
 #include "config.h"
 
+#include <glib/gi18n.h>
+
 #include <gio/gunixinputstream.h>
 #include <gio/gunixoutputstream.h>
 #include <glib-unix.h>
@@ -90,6 +92,11 @@ ide_clang_client_sync_buffers (IdeClangClient *self)
 
   g_assert (IDE_IS_CLANG_CLIENT (self));
 
+  /* Bail if we're in destruction */
+  if (self->state == STATE_SHUTDOWN ||
+      !(context = ide_object_get_context (IDE_OBJECT (self))))
+    return;
+
   /*
    * We need to sync buffers to the subprocess, but only those that are of any
    * consequence to us. So that means C, C++, or Obj-C files and headers.
@@ -103,7 +110,6 @@ ide_clang_client_sync_buffers (IdeClangClient *self)
    * be used on subsequence commands).
    */
 
-  context = ide_object_get_context (IDE_OBJECT (self));
   ufs = ide_unsaved_files_from_context (context);
   ar = ide_unsaved_files_to_array (ufs);
   IDE_PTR_ARRAY_SET_FREE_FUNC (ar, ide_unsaved_file_unref);
@@ -157,6 +163,8 @@ ide_clang_client_subprocess_exited (IdeClangClient          *self,
   g_assert (IDE_IS_SUBPROCESS (subprocess));
   g_assert (IDE_IS_SUBPROCESS_SUPERVISOR (supervisor));
 
+  ide_object_message (self, _("Clang integration server has exited"));
+
   if (self->state == STATE_RUNNING)
     self->state = STATE_SPAWNING;
 
@@ -187,6 +195,10 @@ ide_clang_client_subprocess_spawned (IdeClangClient          *self,
   g_assert (IDE_IS_SUBPROCESS_SUPERVISOR (supervisor));
   g_assert (self->rpc_client == NULL);
 
+  ide_object_message (self,
+                      _("Clang integration server has started as process %s"),
+                      ide_subprocess_get_identifier (subprocess));
+
   if (self->state == STATE_SPAWNING)
     self->state = STATE_RUNNING;
 
@@ -391,8 +403,7 @@ ide_clang_client_destroy (IdeObject *object)
 
   self->state = STATE_SHUTDOWN;
 
-  if (self->seq_by_file != NULL)
-    g_hash_table_remove_all (self->seq_by_file);
+  g_clear_pointer (&self->seq_by_file, g_hash_table_unref);
 
   if (self->supervisor != NULL)
     {
@@ -402,6 +413,7 @@ ide_clang_client_destroy (IdeObject *object)
     }
 
   g_clear_object (&self->rpc_client);
+  g_clear_object (&self->root_uri);
 
   queued = g_steal_pointer (&self->get_client.head);
 
@@ -641,6 +653,7 @@ ide_clang_client_index_file_async (IdeClangClient      *self,
                                    GAsyncReadyCallback  callback,
                                    gpointer             user_data)
 {
+  static const char * const empty[] = { NULL };
   g_autoptr(IdeTask) task = NULL;
   g_autoptr(GVariant) params = NULL;
   g_autofree gchar *path = NULL;
@@ -661,6 +674,9 @@ ide_clang_client_index_file_async (IdeClangClient      *self,
       return;
     }
 
+  if (flags == NULL)
+    flags = empty;
+
   path = g_file_get_path (file);
 
   params = JSONRPC_MESSAGE_NEW (
@@ -687,8 +703,6 @@ ide_clang_client_index_file_async (IdeClangClient      *self,
  *
  * Returns: (transfer full): a #GVariant containing the indexed data
  *   or %NULL in case of failure.
- *
- * Since: 3.32
  */
 GVariant *
 ide_clang_client_index_file_finish (IdeClangClient  *self,
