@@ -286,6 +286,30 @@ ide_clang_completion_provider_activate (GtkSourceCompletionProvider *provider,
         }
     }
 
+  /* If we're in a path context, remove everything within it because
+   * clang will try to replace it and the trailing ".
+   */
+  if (gtk_source_buffer_iter_has_context_class (buffer, &begin, "path") &&
+      gtk_source_buffer_iter_has_context_class (buffer, &end, "path"))
+    {
+      gunichar ch;
+
+      while (gtk_text_iter_backward_char (&begin) &&
+             !gtk_text_iter_starts_line (&begin) &&
+             (ch = gtk_text_iter_get_char (&begin)) &&
+             ch != '/' && ch != '<' && ch != '"')
+        { /* Do Nothing */ }
+
+      end = begin;
+      gtk_source_buffer_iter_forward_to_context_class_toggle (buffer, &end, "path");
+
+      if (gtk_text_iter_get_char (&begin) == '"' ||
+          gtk_text_iter_get_char (&begin) == '<')
+        gtk_text_iter_forward_char (&begin);
+
+      gtk_text_buffer_delete (GTK_TEXT_BUFFER (buffer), &begin, &end);
+    }
+
   if (self->activation_keyval == GDK_KEY_semicolon &&
       !ends_with_string (snippet, ";"))
     {
@@ -306,6 +330,8 @@ ide_clang_completion_provider_refilter (GtkSourceCompletionProvider *provider,
 {
   IdeClangCompletionProvider *self = (IdeClangCompletionProvider *)provider;
 
+  IDE_ENTRY;
+
   g_assert (IDE_IS_CLANG_COMPLETION_PROVIDER (self));
   g_assert (GTK_SOURCE_IS_COMPLETION_CONTEXT (context));
   g_assert (G_IS_LIST_MODEL (proposals));
@@ -319,7 +345,11 @@ ide_clang_completion_provider_refilter (GtkSourceCompletionProvider *provider,
       gtk_source_completion_context_get_bounds (context, &begin, &end);
       self->refilter_word = gtk_text_iter_get_slice (&begin, &end);
       ide_clang_proposals_refilter (self->proposals, self->refilter_word);
+
+      IDE_EXIT;
     }
+
+  IDE_EXIT;
 }
 
 static gchar *
@@ -334,6 +364,8 @@ ide_clang_completion_provider_load (IdeClangCompletionProvider *self)
   g_autoptr(IdeClangClient) client = NULL;
   g_autoptr(IdeContext) context = NULL;
 
+  IDE_ENTRY;
+
   g_assert (IDE_IS_CLANG_COMPLETION_PROVIDER (self));
   g_assert (self->loaded == FALSE);
 
@@ -343,6 +375,8 @@ ide_clang_completion_provider_load (IdeClangCompletionProvider *self)
   client = ide_object_ensure_child_typed (IDE_OBJECT (context), IDE_TYPE_CLANG_CLIENT);
 
   g_set_object (&self->client, client);
+
+  IDE_EXIT;
 }
 
 static void
@@ -354,14 +388,22 @@ ide_clang_completion_provider_populate_cb (GObject      *object,
   g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
 
+  IDE_ENTRY;
+
   g_assert (IDE_IS_CLANG_PROPOSALS (proposals));
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (IDE_IS_TASK (task));
 
   if (!ide_clang_proposals_populate_finish (proposals, result, &error))
-    ide_task_return_error (task, g_steal_pointer (&error));
-  else
-    ide_task_return_pointer (task, g_object_ref (proposals), g_object_unref);
+    {
+      IDE_TRACE_MSG ("Clang completion failed: %s", error->message);
+      ide_task_return_error (task, g_steal_pointer (&error));
+      IDE_EXIT;
+    }
+
+  ide_task_return_pointer (task, g_object_ref (proposals), g_object_unref);
+
+  IDE_EXIT;
 }
 
 static void
@@ -375,6 +417,8 @@ ide_clang_completion_provider_populate_async (GtkSourceCompletionProvider  *prov
   g_autoptr(IdeTask) task = NULL;
   GtkTextIter begin, end;
 
+  IDE_ENTRY;
+
   g_assert (IDE_IS_CLANG_COMPLETION_PROVIDER (self));
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
@@ -408,17 +452,26 @@ ide_clang_completion_provider_populate_async (GtkSourceCompletionProvider  *prov
                                       cancellable,
                                       ide_clang_completion_provider_populate_cb,
                                       g_steal_pointer (&task));
+
+  IDE_EXIT;
 }
 
 static GListModel *
 ide_clang_completion_provider_populate_finish (GtkSourceCompletionProvider  *provider,
-                                               GAsyncResult           *result,
-                                               GError                **error)
+                                               GAsyncResult                 *result,
+                                               GError                      **error)
 {
+  GListModel *ret;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (IDE_IS_CLANG_COMPLETION_PROVIDER (provider));
   g_assert (IDE_IS_TASK (result));
 
-  return ide_task_propagate_pointer (IDE_TASK (result), error);
+  ret = ide_task_propagate_pointer (IDE_TASK (result), error);
+
+  IDE_RETURN (ret);
 }
 
 static void
@@ -462,7 +515,7 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (IdeClangCompletionProvider, ide_clang_completion_
                                G_IMPLEMENT_INTERFACE (GTK_SOURCE_TYPE_COMPLETION_PROVIDER, provider_iface_init))
 
 static void
-ide_clang_completion_provider_dispose (GObject *object)
+ide_clang_completion_provider_destroy (IdeObject *object)
 {
   IdeClangCompletionProvider *self = (IdeClangCompletionProvider *)object;
 
@@ -475,15 +528,15 @@ ide_clang_completion_provider_dispose (GObject *object)
   g_clear_pointer (&self->word, g_free);
   g_clear_pointer (&self->refilter_word, g_free);
 
-  G_OBJECT_CLASS (ide_clang_completion_provider_parent_class)->dispose (object);
+  IDE_OBJECT_CLASS (ide_clang_completion_provider_parent_class)->destroy (object);
 }
 
 static void
 ide_clang_completion_provider_class_init (IdeClangCompletionProviderClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  IdeObjectClass *i_object_class = IDE_OBJECT_CLASS (klass);
 
-  object_class->dispose = ide_clang_completion_provider_dispose;
+  i_object_class->destroy = ide_clang_completion_provider_destroy;
 }
 
 static void
