@@ -1,6 +1,6 @@
 /* ide-clang.c
  *
- * Copyright 2018-2019 Christian Hergert <chergert@redhat.com>
+ * Copyright 2018-2023 Christian Hergert <chergert@redhat.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@ typedef struct
 G_DEFINE_FINAL_TYPE (IdeClang, ide_clang, G_TYPE_OBJECT)
 
 static GHashTable *unsupported_by_clang;
+static GHashTable *auto_suffixes;
 
 static void
 unsaved_files_free (UnsavedFiles *uf)
@@ -538,6 +539,13 @@ ide_clang_class_init (IdeClangClass *klass)
 
   unsupported_by_clang = g_hash_table_new (g_str_hash, g_str_equal);
   g_hash_table_add (unsupported_by_clang, (char *)"-Wstrict-null-sentinel");
+
+  auto_suffixes = g_hash_table_new (g_str_hash, g_str_equal);
+  g_hash_table_add (auto_suffixes, (char *)"_auto");
+  g_hash_table_add (auto_suffixes, (char *)"_autolist");
+  g_hash_table_add (auto_suffixes, (char *)"_autoptr");
+  g_hash_table_add (auto_suffixes, (char *)"_autoqueue");
+  g_hash_table_add (auto_suffixes, (char *)"_autoslist");
 }
 
 static void
@@ -1328,7 +1336,6 @@ ide_clang_build_completion (GVariantBuilder    *builder,
   for (guint i = 0; i < n_chunks; i++)
     {
       g_auto(CXString) str = clang_getCompletionChunkText (result->CompletionString, i);
-      g_autofree char *rewritten = NULL;
       const gchar *text = clang_getCString (str);
       guint kind = clang_getCompletionChunkKind (result->CompletionString, i);
 
@@ -1340,9 +1347,13 @@ ide_clang_build_completion (GVariantBuilder    *builder,
 
           typed_text = g_utf8_casefold (text, -1);
 
-          /* Convert Foo_autoptr into g_autoptr(Foo) */
-          if (bar &&
-              (g_str_equal (bar, "_autoptr") || g_str_equal (bar, "_auto")))
+          /* Convert Foo_autoptr into g_autoptr(Foo) but don't touch
+           * things like "g_autoptr (TypeName)" where we have "g_autotptr"
+           * as the typed text.
+           */
+          if (!g_str_has_prefix (text, "g_auto") &&
+              (bar = strrchr (text, '_')) &&
+              g_hash_table_contains (auto_suffixes, bar))
             {
               GString *string = g_string_new ("g");
               g_string_append (string, bar);
@@ -1350,7 +1361,7 @@ ide_clang_build_completion (GVariantBuilder    *builder,
               g_string_append_len (string, text, strlen (text) - strlen (bar));
               g_string_append_c (string, ')');
               g_variant_builder_add_parsed (&chunks_builder, "{%s,<%s>}", "text", string->str);
-              g_string_free (string, FALSE);
+              g_string_free (string, TRUE);
             }
           else
             {
