@@ -32,17 +32,16 @@ struct _IdeCtagsHighlighter
   IdeObject           parent_instance;
 
   GPtrArray          *indexes;
-  IdeCtagsService    *service;
   IdeHighlightEngine *engine;
 };
 
 static void highlighter_iface_init (IdeHighlighterInterface *iface);
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (IdeCtagsHighlighter,
-                         ide_ctags_highlighter,
-                         IDE_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (IDE_TYPE_HIGHLIGHTER,
-                                                highlighter_iface_init))
+                               ide_ctags_highlighter,
+                               IDE_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (IDE_TYPE_HIGHLIGHTER,
+                                                      highlighter_iface_init))
 
 static inline gboolean
 accepts_char (gunichar ch)
@@ -134,8 +133,27 @@ get_tag (IdeCtagsHighlighter *self,
   return NULL;
 }
 
+static void
+remove_tags (const GtkTextIter *begin,
+             const GtkTextIter *end,
+             const GSList      *tags_to_remove)
+{
+  GtkTextBuffer *buffer;
+
+  g_assert (begin != NULL);
+  g_assert (end != NULL);
+
+  if (tags_to_remove == NULL)
+    return;
+
+  buffer = gtk_text_iter_get_buffer (begin);
+  for (const GSList *iter = tags_to_remove; iter; iter = iter->next)
+    gtk_text_buffer_remove_tag (buffer, iter->data, begin, end);
+}
+
 static void
 ide_ctags_highlighter_real_update (IdeHighlighter       *highlighter,
+                                   const GSList         *tags_to_remove,
                                    IdeHighlightCallback  callback,
                                    const GtkTextIter    *range_begin,
                                    const GtkTextIter    *range_end,
@@ -165,8 +183,15 @@ ide_ctags_highlighter_real_update (IdeHighlighter       *highlighter,
 
   while (gtk_text_iter_compare (&begin, range_end) < 0)
     {
+      GtkTextIter last = begin;
+
       if (!select_next_word (&begin, &end))
-        goto completed;
+        {
+          remove_tags (&last, range_end, tags_to_remove);
+          goto completed;
+        }
+
+      remove_tags (&last, &end, tags_to_remove);
 
       if (gtk_text_iter_compare (&begin, range_end) >= 0)
         goto completed;
@@ -254,10 +279,7 @@ ide_ctags_highlighter_real_set_engine (IdeHighlighter      *highlighter,
 
   if ((context = ide_object_get_context (IDE_OBJECT (self))) &&
       (service = ide_object_get_child_typed (IDE_OBJECT (context), IDE_TYPE_CTAGS_SERVICE)))
-    {
-      g_set_weak_pointer (&self->service, service);
-      ide_ctags_service_register_highlighter (service, self);
-    }
+    ide_ctags_service_register_highlighter (service, self);
 }
 
 static void
@@ -265,12 +287,6 @@ ide_ctags_highlighter_finalize (GObject *object)
 {
   IdeCtagsHighlighter *self = (IdeCtagsHighlighter *)object;
 
-  if (self->service != NULL)
-    {
-      ide_ctags_service_unregister_highlighter (self->service, self);
-      g_clear_weak_pointer (&self->service);
-    }
-
   g_clear_pointer (&self->indexes, g_ptr_array_unref);
 
   G_OBJECT_CLASS (ide_ctags_highlighter_parent_class)->finalize (object);
