@@ -119,8 +119,8 @@ language_to_label (GBinding     *binding,
   if (language != NULL)
     g_value_set_string (to_value, gtk_source_language_get_name (language));
   else
-    /* translators: "Text" means plaintext or text/plain */
-    g_value_set_static_string (to_value, _("Text"));
+    /* translators: "Plain Text" means text without formatting */
+    g_value_set_static_string (to_value, _("Plain Text"));
 
   return TRUE;
 }
@@ -213,10 +213,10 @@ update_position (GbpEditoruiWorkspaceAddin *self)
 
   if ((view = ide_signal_group_get_target (self->view_signals)))
     {
-      guint line, column;
+      guint line, column, range;
 
-      ide_source_view_get_visual_position (view, &line, &column);
-      gbp_editorui_position_label_update (self->position_label, line, column);
+      ide_source_view_get_visual_position_range (view, &line, &column, &range);
+      gbp_editorui_position_label_update (self->position_label, line, column, range);
     }
 }
 
