@@ -514,7 +514,7 @@ gbp_flatpak_runtime_set_branch (GbpFlatpakRuntime *self,
 {
   g_return_if_fail (GBP_IS_FLATPAK_RUNTIME (self));
 
-  if (ide_set_string (&self->branch, branch))
+  if (g_set_str (&self->branch, branch))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_BRANCH]);
 }
 
@@ -532,7 +532,7 @@ gbp_flatpak_runtime_set_platform (GbpFlatpakRuntime *self,
 {
   g_return_if_fail (GBP_IS_FLATPAK_RUNTIME (self));
 
-  if (ide_set_string (&self->platform, platform))
+  if (g_set_str (&self->platform, platform))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_PLATFORM]);
 }
 
@@ -568,10 +568,8 @@ gbp_flatpak_runtime_set_sdk (GbpFlatpakRuntime *self,
 {
   g_return_if_fail (GBP_IS_FLATPAK_RUNTIME (self));
 
-  if (g_strcmp0 (sdk, self->sdk) != 0)
+  if (g_set_str (&self->sdk, sdk))
     {
-      g_free (self->sdk);
-      self->sdk = g_strdup (sdk);
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SDK]);
     }
 }
@@ -748,8 +746,7 @@ gbp_flatpak_runtime_init (GbpFlatpakRuntime *self)
 }
 
 GbpFlatpakRuntime *
-gbp_flatpak_runtime_new (IdeObject  *parent,
-                         const char *name,
+gbp_flatpak_runtime_new (const char *name,
                          const char *arch,
                          const char *branch,
                          const char *sdk_name,
@@ -761,10 +758,10 @@ gbp_flatpak_runtime_new (IdeObject  *parent,
   g_autofree char *id = NULL;
   g_autofree char *short_id = NULL;
   g_autofree char *triplet = NULL;
+  g_autofree char *system_arch = NULL;
   g_autoptr(IdeTriplet) triplet_object = NULL;
   g_autoptr(GString) category = NULL;
 
-  g_return_val_if_fail (IDE_IS_OBJECT (parent), NULL);
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail (arch != NULL, NULL);
   g_return_val_if_fail (branch != NULL, NULL);
@@ -790,13 +787,14 @@ gbp_flatpak_runtime_new (IdeObject  *parent,
   else if (g_str_has_prefix (name, "org.kde."))
     g_string_append (category, "KDE/");
 
-  if (ide_str_equal0 (ide_get_system_arch (), arch))
+  system_arch = ide_get_system_arch ();
+
+  if (ide_str_equal0 (system_arch, arch))
     g_string_append (category, name);
   else
     g_string_append_printf (category, "%s (%s)", name, arch);
 
   return g_object_new (GBP_TYPE_FLATPAK_RUNTIME,
-                       "parent", parent,
                        "id", id,
                        "short-id", short_id,
                        "triplet", triplet_object,
