@@ -29,6 +29,7 @@
 #include "gbp-meson-build-system.h"
 #include "gbp-meson-build-target.h"
 #include "gbp-meson-toolchain.h"
+#include "gbp-meson-utils.h"
 
 struct _GbpMesonBuildSystem
 {
@@ -704,6 +705,33 @@ gbp_meson_build_system_get_builddir (IdeBuildSystem   *build_system,
   return NULL;
 }
 
+static char *
+gbp_meson_build_system_get_srcdir (IdeBuildSystem *build_system)
+{
+  GbpMesonBuildSystem *self = (GbpMesonBuildSystem *)build_system;
+  g_autoptr(GFile) workdir = NULL;
+  g_autofree char *base = NULL;
+  IdeContext *context;
+
+  g_assert (GBP_IS_MESON_BUILD_SYSTEM (self));
+
+  context = ide_object_get_context (IDE_OBJECT (self));
+  workdir = ide_context_ref_workdir (context);
+
+  if (self->project_file == NULL)
+    return g_strdup (g_file_peek_path (workdir));
+
+  base = g_file_get_basename (self->project_file);
+
+  if (strcasecmp (base, "meson.build") == 0)
+    {
+      g_autoptr(GFile) parent = g_file_get_parent (self->project_file);
+      return g_file_get_path (parent);
+    }
+
+  return g_file_get_path (self->project_file);
+}
+
 static gboolean
 gbp_meson_build_system_supports_toolchain (IdeBuildSystem *self,
                                            IdeToolchain   *toolchain)
@@ -743,6 +771,104 @@ gbp_meson_build_system_supports_language (IdeBuildSystem *system,
   return FALSE;
 }
 
+static gboolean
+gbp_meson_build_system_prepare_tooling_cb (IdeRunContext       *run_context,
+                                           const char * const  *argv,
+                                           const char * const  *env,
+                                           const char          *cwd,
+                                           IdeUnixFDMap        *unix_fd_map,
+                                           gpointer             user_data,
+                                           GError             **error)
+{
+  const char *devenv_file = user_data;
+  g_autofree char *devenv_file_quoted = NULL;
+  g_autoptr(GString) argv0 = NULL;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_RUN_CONTEXT (run_context));
+  g_assert (devenv_file != NULL);
+
+  if (!ide_run_context_merge_unix_fd_map (run_context, unix_fd_map, error))
+    return FALSE;
+
+  devenv_file_quoted = g_shell_quote (devenv_file);
+
+  argv0 = g_string_new (NULL);
+  g_string_append_printf (argv0, ". %s\n", devenv_file_quoted);
+
+  if (!ide_str_empty0 (cwd))
+    {
+      g_autofree char *quoted = g_shell_quote (cwd);
+      g_string_append_printf (argv0, "cd %s\n", quoted);
+    }
+
+  if (env != NULL && env[0] != NULL)
+    {
+      g_string_append (argv0, "env ");
+
+      for (guint i = 0; env[i]; i++)
+        {
+          g_autofree char *quoted = g_shell_quote (env[i]);
+
+          g_string_append (argv0, quoted);
+          g_string_append_c (argv0, ' ');
+        }
+    }
+
+  for (guint i = 0; argv[i]; i++)
+    {
+      g_autofree char *quoted = g_shell_quote (argv[i]);
+
+      g_string_append (argv0, quoted);
+      g_string_append_c (argv0, ' ');
+    }
+
+  g_string_append (argv0, "\n");
+
+  ide_run_context_set_argv (run_context, IDE_STRV_INIT ("/bin/sh", "-c", argv0->str));
+
+  IDE_RETURN (TRUE);
+}
+
+static void
+gbp_meson_build_system_prepare_tooling (IdeBuildSystem *build_system,
+                                        IdeRunContext  *run_context)
+{
+  GbpMesonBuildSystem *self = (GbpMesonBuildSystem *)build_system;
+  g_autofree char *devenv_file = NULL;
+  IdeBuildManager *build_manager;
+  IdePipeline *pipeline;
+  IdeContext *context;
+  const char *builddir;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (GBP_IS_MESON_BUILD_SYSTEM (self));
+  g_assert (IDE_IS_RUN_CONTEXT (run_context));
+
+  if ((context = ide_object_get_context (IDE_OBJECT (self))) &&
+      (build_manager = ide_build_manager_from_context (context)) &&
+      (pipeline = ide_build_manager_get_pipeline (build_manager)) &&
+      (builddir = ide_pipeline_get_builddir (pipeline)) &&
+      (devenv_file = g_build_filename (builddir, ".gnome-builder-devenv", NULL)) &&
+      gbp_meson_devenv_sanity_check (devenv_file))
+    {
+      ide_run_context_push (run_context,
+                            gbp_meson_build_system_prepare_tooling_cb,
+                            g_steal_pointer (&devenv_file),
+                            g_free);
+
+      IDE_EXIT;
+    }
+
+  g_debug ("Pipeline is not configured far enough to use meson devenv");
+
+  IDE_EXIT;
+}
+
 static void
 build_system_iface_init (IdeBuildSystemInterface *iface)
 {
@@ -754,7 +880,9 @@ build_system_iface_init (IdeBuildSystemInterface *iface)
   iface->get_build_flags_for_files_async = gbp_meson_build_system_get_build_flags_for_files_async;
   iface->get_build_flags_for_files_finish = gbp_meson_build_system_get_build_flags_for_files_finish;
   iface->get_builddir = gbp_meson_build_system_get_builddir;
+  iface->get_srcdir = gbp_meson_build_system_get_srcdir;
   iface->get_project_version = gbp_meson_build_system_get_project_version;
+  iface->prepare_tooling = gbp_meson_build_system_prepare_tooling;
   iface->supports_toolchain = gbp_meson_build_system_supports_toolchain;
   iface->supports_language = gbp_meson_build_system_supports_language;
 }
@@ -872,7 +1000,7 @@ extract_metadata (GbpMesonBuildSystem *self,
   g_regex_match (regex, contents, 0, &match_info);
   while (g_match_info_matches (match_info))
     {
-      const gchar *str = g_match_info_fetch (match_info, 1);
+      g_autofree gchar *str = g_match_info_fetch (match_info, 1);
       self->languages = _gbp_meson_build_system_parse_languages (str);
 
       g_match_info_next (match_info, NULL);
@@ -987,42 +1115,24 @@ gbp_meson_build_system_get_languages (GbpMesonBuildSystem *self)
   return (const gchar * const *)self->languages;
 }
 
-char *
-gbp_meson_build_system_get_project_dir (GbpMesonBuildSystem *self)
-{
-  g_autoptr(GFile) workdir = NULL;
-  g_autofree char *base = NULL;
-  IdeContext *context;
-
-  g_return_val_if_fail (GBP_IS_MESON_BUILD_SYSTEM (self), NULL);
-
-  context = ide_object_get_context (IDE_OBJECT (self));
-  workdir = ide_context_ref_workdir (context);
-
-  if (self->project_file == NULL)
-    return g_strdup (g_file_peek_path (workdir));
-
-  base = g_file_get_basename (self->project_file);
-
-  if (strcasecmp (base, "meson.build") == 0)
-    {
-      g_autoptr(GFile) parent = g_file_get_parent (self->project_file);
-      return g_file_get_path (parent);
-    }
-
-  return g_file_get_path (self->project_file);
-}
-
 char *
 gbp_meson_build_system_locate_meson (GbpMesonBuildSystem *self,
                                      IdePipeline         *pipeline)
 {
+  IdeContext *context;
   IdeConfig *config;
 
   g_return_val_if_fail (!self || GBP_IS_MESON_BUILD_SYSTEM (self), NULL);
   g_return_val_if_fail (!pipeline || IDE_IS_PIPELINE (pipeline), NULL);
 
-  if ((config = ide_pipeline_get_config (pipeline)))
+  context = ide_object_get_context (IDE_OBJECT (self));
+
+  if (pipeline == NULL)
+    config = ide_config_manager_get_current (ide_config_manager_from_context (context));
+  else
+    config = ide_pipeline_get_config (pipeline);
+
+  if (config != NULL)
     {
       const char *envvar = ide_config_getenv (config, "MESON");
 
