@@ -31,6 +31,7 @@
 #include "gbp-meson-build-target.h"
 #include "gbp-meson-introspection.h"
 #include "gbp-meson-pipeline-addin.h"
+#include "gbp-meson-utils.h"
 
 struct _GbpMesonPipelineAddin
 {
@@ -138,7 +139,6 @@ attach_run_command (GbpMesonPipelineAddin *self,
                     IdePipelinePhase       phase)
 {
   g_autoptr(IdePipelineStage) stage = NULL;
-  g_autoptr(GError) error = NULL;
   guint id;
 
   g_assert (GBP_IS_MESON_PIPELINE_ADDIN (self));
@@ -156,11 +156,58 @@ attach_run_command (GbpMesonPipelineAddin *self,
   return stage;
 }
 
+static int
+is_newer (const char *old,
+          const char *new)
+{
+  g_autoptr(GFile) file_a = g_file_new_for_path (old);
+  g_autoptr(GFile) file_b = g_file_new_for_path (new);
+  g_autoptr(GFileInfo) info_a = g_file_query_info (file_a, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
+  g_autoptr(GFileInfo) info_b = g_file_query_info (file_b, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL);
+  guint64 mtime_a;
+  guint64 mtime_b;
+
+  if (info_a == NULL || info_b == NULL)
+    return FALSE;
+
+  mtime_a = g_file_info_get_attribute_uint64 (info_a, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+  mtime_b = g_file_info_get_attribute_uint64 (info_b, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+
+  return mtime_b > mtime_a;
+}
+
+static void
+devenv_query_cb (IdePipelineStage *stage,
+                 IdePipeline      *pipeline,
+                 GPtrArray        *targets,
+                 GCancellable     *cancellable)
+{
+  g_autofree char *devenv_file = NULL;
+  g_autofree char *build_ninja = NULL;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_PIPELINE_STAGE (stage));
+  g_assert (IDE_IS_PIPELINE (pipeline));
+  g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
+
+  devenv_file = ide_pipeline_build_builddir_path (pipeline, ".gnome-builder-devenv", NULL);
+  build_ninja = ide_pipeline_build_builddir_path (pipeline, "build.ninja", NULL);
+
+  /* If the build.ninja is newer than our devenv file, it needs to be
+   * regenerated to get updated configuration.
+   */
+  if (!is_newer (build_ninja, devenv_file) ||
+      !gbp_meson_devenv_sanity_check (devenv_file))
+    ide_pipeline_stage_set_completed (stage, FALSE);
+}
+
 static void
 gbp_meson_pipeline_addin_load (IdePipelineAddin *addin,
                                IdePipeline      *pipeline)
 {
   GbpMesonPipelineAddin *self = (GbpMesonPipelineAddin *)addin;
+  g_autoptr(IdeSubprocessLauncher) devenv_launcher = NULL;
+  g_autoptr(IdePipelineStage) devenv_stage = NULL;
   g_autoptr(IdeRunCommand) build_command = NULL;
   g_autoptr(IdeRunCommand) clean_command = NULL;
   g_autoptr(IdeRunCommand) config_command = NULL;
@@ -168,6 +215,7 @@ gbp_meson_pipeline_addin_load (IdePipelineAddin *addin,
   IdePipelineStage *stage;
   g_autofree char *build_dot_ninja = NULL;
   g_autofree char *crossbuild_file = NULL;
+  g_autofree char *devenv_file = NULL;
   g_autofree char *meson = NULL;
   g_autofree char *ninja = NULL;
   IdeBuildSystem *build_system;
@@ -252,6 +300,23 @@ gbp_meson_pipeline_addin_load (IdePipelineAddin *addin,
                             IDE_PIPELINE_STAGE (self->introspection));
   ide_pipeline_addin_track (addin, id);
 
+  /* Setup stage to extract "devenv" settings */
+  devenv_file = ide_pipeline_build_builddir_path (pipeline, ".gnome-builder-devenv", NULL);
+  devenv_launcher = ide_subprocess_launcher_new (0);
+  ide_subprocess_launcher_set_stdout_file_path (devenv_launcher, devenv_file);
+  ide_subprocess_launcher_push_args (devenv_launcher, IDE_STRV_INIT ("meson", "devenv", "--dump"));
+  ide_subprocess_launcher_set_cwd (devenv_launcher, ide_pipeline_get_builddir (pipeline));
+  devenv_stage = ide_pipeline_stage_launcher_new (ide_object_get_context (IDE_OBJECT (pipeline)), devenv_launcher);
+  ide_pipeline_stage_launcher_set_use_pty (IDE_PIPELINE_STAGE_LAUNCHER (devenv_stage), FALSE);
+  ide_pipeline_stage_launcher_set_ignore_exit_status (IDE_PIPELINE_STAGE_LAUNCHER (devenv_stage), TRUE);
+  ide_pipeline_stage_set_name (devenv_stage, _("Cache development environment"));
+  g_signal_connect (devenv_stage, "query", G_CALLBACK (devenv_query_cb), NULL);
+  id = ide_pipeline_attach (pipeline,
+                            IDE_PIPELINE_PHASE_CONFIGURE | IDE_PIPELINE_PHASE_AFTER,
+                            0,
+                            devenv_stage);
+  ide_pipeline_addin_track (addin, id);
+
   IDE_EXIT;
 }
 
@@ -265,21 +330,21 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (GbpMesonPipelineAddin, gbp_meson_pipeline_addin,
                                G_IMPLEMENT_INTERFACE (IDE_TYPE_PIPELINE_ADDIN, pipeline_addin_iface_init))
 
 static void
-gbp_meson_pipeline_addin_dispose (GObject *object)
+gbp_meson_pipeline_addin_destroy (IdeObject *object)
 {
   GbpMesonPipelineAddin *self = (GbpMesonPipelineAddin *)object;
 
   g_clear_object (&self->introspection);
 
-  G_OBJECT_CLASS (gbp_meson_pipeline_addin_parent_class)->dispose (object);
+  IDE_OBJECT_CLASS (gbp_meson_pipeline_addin_parent_class)->destroy (object);
 }
 
 static void
 gbp_meson_pipeline_addin_class_init (GbpMesonPipelineAddinClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  IdeObjectClass *i_object_class = IDE_OBJECT_CLASS (klass);
 
-  object_class->dispose = gbp_meson_pipeline_addin_dispose;
+  i_object_class->destroy = gbp_meson_pipeline_addin_destroy;
 }
 
 static void
